"use strict";
/*----------------------------------
- DEPENDENCIES
----------------------------------*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
// Npm
var get_root_domain_1 = __importDefault(require("get-root-domain"));
var isURL_1 = __importDefault(require("validator/lib/isURL"));
// Interval
var types_1 = require("./types");
var BadRequest = /** @class */ (function (_super) {
    __extends(BadRequest, _super);
    function BadRequest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return BadRequest;
}(Error));
/*----------------------------------
- CONFIG
----------------------------------*/
var reqPerCall = 3;
/*----------------------------------
- METHODS
----------------------------------*/
exports["default"] = (function (requests) {
    // TODO: Check remaining requests / minute
    // Type Check
    if (!Array.isArray(requests))
        throw new BadRequest("requests must be an array. Provided: " + typeof requests);
    // Requests number / call
    var reqCount = requests.length;
    if (reqCount === 0)
        throw new BadRequest("You must provide at least one request.");
    if (reqCount > reqPerCall)
        throw new BadRequest("You can't send more than " + reqPerCall + " requests per call (" + reqCount + " given).");
    // Check every request
    var domains = {};
    for (var iReq = 0; iReq < reqCount; iReq++) {
        var req = requests[iReq];
        // Type
        if (typeof req !== "object" || req === null)
            throw new BadRequest("requests must be an array of requests object, but requests[" + iReq + "] is an " + typeof req);
        // Method
        if (req.method === undefined)
            req.method = 'GET';
        else if (!types_1.allowedMethods.includes(req.method))
            throw new BadRequest("Only the following HTTP methods are currently allowed: " + types_1.allowedMethods.join(', '));
        // URL
        if (typeof req.url !== "string" || !(0, isURL_1["default"])(req.url, {
            require_protocol: true,
            require_valid_protocol: true,
            protocols: ['http', 'https'],
            require_host: true,
            require_port: false,
            allow_protocol_relative_urls: false
        }))
            throw new BadRequest("The url parameter must be a valid URL string: protocol (http or https) + domain + path (optional)");
        // Unique domain
        if (reqCount !== 1) {
            var domain = (0, get_root_domain_1["default"])(req.url);
            if (domains[domain] === true)
                throw new BadRequest("When you send multiple requests in one call, each requets must point to different domain names. However, you're sending 2 requests to " + domain + ".");
            domains[domain] = true;
        }
        // Cookies
        if (req.cookies !== undefined) {
            // Type
            if (typeof req.cookies !== 'string')
                throw new BadRequest("The cookie parameter must be a string. Example: user=Bob; age=28;");
        }
        // body
        if (req.body !== undefined) {
            // Bodytype
            if (req.bodyType === undefined)
                throw new BadRequest("The bodyType parameter must be provided when the body parameter is specified.");
            if (!types_1.bodyTypes.includes(req.bodyType))
                throw new BadRequest("Invalid value for the bodyType parameter. Allowed values: " + types_1.bodyTypes.join(', '));
            // Type
            if (typeof req.body !== 'object' || req.body.constructor.name !== 'Object')
                throw new BadRequest("The body parameter must be an object.");
        }
        if (req.extract !== undefined)
            validateExtractors(req.extract, 'extract');
        if (req.withBody !== undefined && typeof req.withBody !== "boolean")
            throw new BadRequest("The withBody parameter must be a boolean.");
        if (req.withHeaders !== undefined && typeof req.withHeaders !== "boolean")
            throw new BadRequest("The withHeaders parameter must be a boolean.");
    }
    return requests;
});
var validateExtractors = function (extract, path) {
    if (!extract || typeof extract !== 'object')
        throw new BadRequest("The " + path + " option must be an object or an array (" + typeof extract + " given).");
    if (Array.isArray(extract)) {
        /*
            extract: ["h4", "text", true, "title"]
        */
        if (extract.length < 3 || typeof extract[0] !== "string" || typeof extract[1] !== "string" || typeof extract[2] !== "boolean")
            throw new BadRequest("When the " + path + " option is an array, it must contain at least 3 values: "
                + "CSS selector (string), attribute (string), required (boolean) and optionnaly filters (strings)");
    }
    else {
        /*
            extract: {
                "$foreach": ".sh-dgr__content",
                "name": ["h4", "text", true, "title"],
                ...
            }
        */
        if (Object.keys(extract).length === 0)
            throw new BadRequest("The " + path + " parameter must contain at least one entry.");
        // @ts-ignore: Property '$foreach' does not exist on type 'TExtractor'
        var _a = extract, $foreach = _a.$foreach, toExtract = __rest(_a, ["$foreach"]);
        // Foreach
        if ($foreach !== undefined && typeof $foreach !== "string")
            throw new BadRequest("When specified, the " + path + ".$foreach parameter must be a CSS selector string.");
        // For each extractor
        for (var name_1 in toExtract) {
            validateExtractors(toExtract[name_1], path + '.' + name_1);
        }
    }
    return extract;
};
